%%%%%%%%%%%%%%%%%%  Optimal sensing matrix based on Theorem3
%%% "On Projection Matrix Optimization for Compressive Sensing Systems,"
%%%  IEEE Transactions on Signal Processing 2013.  
%%%  by G. Li, Z. Zhu, D. Yang, L. Chang, and H. Bai


%%%%%%%%% Input:
% Psi - dictionary given of dimension N x L
% M - no. of rows in the sensing matrix to be designed


function[Phi]= OptSensing(Psi,M,N_iter)


% setup for generating Phi from V_11
[N, L]=size(Psi);  N_r=rank(Psi);   
LB = sqrt((L-M)/(M*(L-1)))*1;   
[U_d,S_d,V_d]=svd(Psi);
invS = diag(1./diag(S_d(1:N_r,1:N_r)));
PhiV = @(V_11) V_11(:,1:M)'*invS*U_d';  %%%%%%% optimal sensing matrix: || I_L - Psi'*X'*X*Psi ||_F

Phi = PhiV(eye(N_r)); % initialization
for k=1:N_iter
  %%%%%%%% Step I: Shrinking
  PhiPsi = normc(Phi*Psi);
  G_n = PhiPsi'*PhiPsi;
  pos=find(abs(G_n(:))>LB & abs(G_n(:)-1)>1e-6);
  G_n(pos)=sign(G_n(pos))*LB;    %%%%%%%%  updating:   Sheringkaged Gram
  %%%%%%%%%% Step II: optimizing V_11
  %V_d=D_sc*V_d;
  Q=V_d'*G_n*V_d; Q_11=Q(1:N_r,1:N_r);
  [V_e,D_e]=eig(Q_11); [~,index] = sort(diag(D_e),'descend'); 
  % formulate Phi
  Phi =  PhiV(V_e(:,index(1:N_r)));  
end



