%%%%%%%%%%%%%%%%%% Demo for fig. 1 and fig.4 in
%%% "On Projection Matrix Optimization for Compressive Sensing Systems,"
%%%  IEEE Transactions on Signal Processing 2013.  
%%%  by G. Li, Z. Zhu, D. Yang, L. Chang, and H. Bai

close all; clear all;
%% seting the parameters
M = 25; N = 80; L = 120;
Psi=normc(randn(N,L));     %dictionary
Phi=randn(M,N);      % initializing the sensing matrix
J=1000; % number of examples
%% optimization of the sensing matrix
Phi_opt = OptSensing(Psi,M,1000);
%% generate the sparse signal A
K=[1:1:7];
RESULT=[];i=1;A=zeros(L,J,length(K));
for s=K
    A_=zeros(L,J); A_=sparse(A_);
    A(:,:,i)=A_;
    h=waitbar(0,'Building the sparse representations ...');
    for j=1:1:J
        waitbar(j/J);
        pos=randperm(L);
        pos=pos(1:s);
        A(pos,j,i)=randn(s,1);
    end;
    close(h);
    i=i+1;
end

%%  decoding use differenet kinds of sensing matrix

for i=1:length(K)
    [ef_random(i) er_random(i)] = simult_sparse_code(Phi,Psi,A(:,:,i),K(i));
    [ef_opt(i) er_opt(i)]= simult_sparse_code(Phi_opt,Psi,A(:,:,i),K(i));
end

%% RESULTS
figure
box on
semilogy(K,ef_random.*(ef_random>=1e-5),'*-.k','Linewidth',2); hold on;
semilogy(K,ef_opt.*(ef_opt>=1e-5),'+--r','Linewidth',2);hold on;
xlabel('\it K');set(gca,'xtick',K); 
ylabel('Reconstruction error');
legend('random ','optimized','Location','Best');

figure
box on
semilogy(K,er_random,'*-.k','Linewidth',2); hold on;
semilogy(K,er_opt,'+--r','Linewidth',2);hold on;
xlabel('\it K');set(gca,'xtick',K); 
ylabel('Reconstruction error rate');
legend('random ','optimized','Location','Best');

% figure the histogram
XX=500;
figure
G = normc(Phi*Psi)'*normc(Phi*Psi);
gg=sort(abs(G(:)));
gg2=gg(1:1:end-L);
[a1,b1]=hist(gg2,0:0.01:1);
subplot(2,1,1);
bar(b1,a1,'b');
axis([0 1 0 XX]);
text(0.5,XX/2,'\fontsize{14}\it random matrix')

G = normc(Phi_opt*Psi)'*normc(Phi_opt*Psi);
gg=sort(abs(G(:)));
gg2=gg(1:1:end-L);
[a1,b1]=hist(gg2,0:0.01:1);
subplot(2,1,2);
bar(b1,a1,'b');
axis([0 1 0 XX]);
text(0.5,XX/2,'\fontsize{14}\it optimized matrix')